Library IEEE;
Use     IEEE.Std_Logic_1164.All;

Entity Configurable_ReplaceWith_Designator Is
Port (

    MEM_D           : InOut Std_Logic_Vector(31 Downto 0);
    MEM_A           : Out   Std_Logic_Vector(23 Downto 0);
    MEM_OE          : Out   Std_Logic;
    MEM_W           : Out   Std_Logic;
    MEM_BE          : Out   Std_Logic_Vector(3 Downto 0);
    MEM_FLASH_E     : Out   Std_Logic;
    MEM_FLASH_BUSY  : In    Std_Logic;
    MEM_FLASH_RESET : Out   Std_Logic;
    MEM_SRAM_E      : Out   Std_Logic;
    MEM_SDRAM_CKE   : Out   Std_Logic;
    MEM_SDRAM_RAS   : Out   Std_Logic;
    MEM_SDRAM_CAS   : Out   Std_Logic;
    MEM_SDRAM_E     : Out   Std_Logic;
    MEM_USB_E       : Out   Std_Logic;
    MEM_USB_RESET   : Out   Std_Logic;

    ---------------------------------------------------------------------------
    -- Wishbone Slave Ports
    ---------------------------------------------------------------------------
    FLASH_CLK_I      : In    Std_Logic;
    FLASH_RST_I      : In    Std_Logic;
    FLASH_CYC_I      : In    Std_Logic;
    FLASH_STB_I      : In    Std_Logic;
    FLASH_ACK_O      : Out   Std_Logic;
    FLASH_ADR_I      : In    Std_Logic_Vector(ReplaceWith_FlashInputAddressWidth - 1 DownTo 0);
    FLASH_DAT_I      : In    Std_Logic_Vector(31 DownTo 0);
    FLASH_DAT_O      : Out   Std_Logic_Vector(31 DownTo 0);
    FLASH_WE_I       : In    Std_Logic;
    FLASH_SEL_I      : In    Std_Logic_Vector( 3 DownTo 0);

    USB_CLK_I      : In    Std_Logic;
    USB_RST_I      : In    Std_Logic;
    USB_CYC_I      : In    Std_Logic;
    USB_STB_I      : In    Std_Logic;
    USB_ACK_O      : Out   Std_Logic;
    USB_ADR_I      : In    Std_Logic_Vector(ReplaceWith_UsbInputAddressWidth-1 DownTo 0);
    USB_DAT_I      : In    Std_Logic_Vector(31 DownTo 0);
    USB_DAT_O      : Out   Std_Logic_Vector(31 DownTo 0);
    USB_WE_I       : In    Std_Logic;
    USB_SEL_I      : In    Std_Logic_Vector(3 DownTo 0)
);
End Configurable_ReplaceWith_Designator;

Architecture RTL Of Configurable_ReplaceWith_Designator Is

    ----------------------------------------------------------------------------
    Component Configurable_ReplaceWith_Designator_Usb
    Port (
         MEM_D_O         : Out   Std_Logic_Vector(31 Downto 0);
         MEM_D_I         : In    Std_Logic_Vector(31 Downto 0);
         MEM_D_E         : Out   Std_Logic;
         MEM_A           : Out   Std_Logic_Vector(23 DownTo 0);
         MEM_OE          : Out   Std_Logic;
         MEM_W           : Out   Std_Logic;
         MEM_SRAM_E      : Out   Std_Logic;
         MEM_BE          : Out   Std_Logic_Vector(3 Downto 0);
         ---------------------------------------------------------------------------
         MEM_SDRAM_CKE   : Out   Std_Logic;
         MEM_SDRAM_RAS   : Out   Std_Logic;
         MEM_SDRAM_CAS   : Out   Std_Logic;
         MEM_SDRAM_E     : Out   Std_Logic;
         MEM_FLASH_E     : Out   Std_Logic;
         MEM_FLASH_BUSY  : In    Std_Logic;
         MEM_FLASH_RESET : Out   Std_Logic;
         MEM_USB_E       : Out   Std_Logic;
         MEM_USB_RESET   : Out   Std_Logic;
         ---------------------------------------------------------------------------
         USB_CLK_I      : In    Std_Logic;
         USB_RST_I      : In    Std_Logic;
         USB_CYC_I      : In    Std_Logic;
         USB_STB_I      : In    Std_Logic;
         USB_ACK_O      : Out   Std_Logic;
         USB_ADR_I      : In    Std_Logic_Vector(ReplaceWith_UsbInputAddressWidth-1 DownTo 0);
         USB_DAT_I      : In    Std_Logic_Vector(31 DownTo 0);
         USB_DAT_O      : Out   Std_Logic_Vector(31 DownTo 0);
         USB_WE_I       : In    Std_Logic;
         USB_SEL_I      : In    Std_Logic_Vector(3 DownTo 0)
    );
    End Component;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Component Configurable_ReplaceWith_Designator_Flash
    Port (
         -----------------------------------------------------------------------------
         MEM_D_O         : Out   Std_Logic_Vector(31 Downto 0);
         MEM_D_I         : In    Std_Logic_Vector(31 Downto 0);
         MEM_D_E         : Out   Std_Logic;
         MEM_A           : Out    Std_Logic_Vector(23 DownTo 0);
         MEM_OE          : Out    Std_Logic;
         MEM_W           : Out    Std_Logic;
         MEM_FLASH_E     : Out    Std_Logic;
         MEM_FLASH_BUSY  : In     Std_Logic;
         MEM_FLASH_RESET : Out    Std_Logic;
         -----------------------------------------------------------------------------
         MEM_SDRAM_CKE   : Out    Std_Logic;
         MEM_SDRAM_RAS   : Out    Std_Logic;
         MEM_SDRAM_CAS   : Out    Std_Logic;
         MEM_SDRAM_E     : Out    Std_Logic;
         MEM_SRAM_E      : Out    Std_Logic;
         MEM_BE          : Out    Std_Logic_Vector(3 Downto 0);
         MEM_USB_E       : Out   Std_Logic;
         MEM_USB_RESET   : Out   Std_Logic;
         ---------------------------------------------------------------------------
         FLASH_CLK_I      : In    Std_Logic;
         FLASH_RST_I      : In    Std_Logic;
         FLASH_CYC_I      : In    Std_Logic;
         FLASH_STB_I      : In    Std_Logic;
         FLASH_ACK_O      : Out   Std_Logic;
         FLASH_ADR_I      : In    Std_Logic_Vector(ReplaceWith_FlashInputAddressWidth - 1 DownTo 0);
         FLASH_DAT_I      : In    Std_Logic_Vector(31 DownTo 0);
         FLASH_DAT_O      : Out   Std_Logic_Vector(31 DownTo 0);
         FLASH_WE_I       : In    Std_Logic;
         FLASH_SEL_I      : In    Std_Logic_Vector( 3 DownTo 0)
    );
    End Component;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Signal MEM_D_O_FromUsb           : Std_Logic_Vector(31 Downto 0);
    Signal MEM_D_E_FromUsb           : Std_Logic;
    Signal MEM_A_FromUsb             : Std_Logic_Vector(23 Downto 0);
    Signal MEM_OE_FromUsb            : Std_Logic;
    Signal MEM_W_FromUsb             : Std_Logic;
    Signal MEM_SRAM_E_FromUsb        : Std_Logic;
    Signal MEM_BE_FromUsb            : Std_Logic_Vector(3 Downto 0);
    Signal MEM_SDRAM_CKE_FromUsb     : Std_Logic;
    Signal MEM_SDRAM_RAS_FromUsb     : Std_Logic;
    Signal MEM_SDRAM_CAS_FromUsb     : Std_Logic;
    Signal MEM_SDRAM_E_FromUsb       : Std_Logic;
    Signal MEM_FLASH_E_FromUsb       : Std_Logic;
    Signal MEM_FLASH_RESET_FromUsb   : Std_Logic;
    Signal MEM_USB_E_FromUsb         : Std_Logic;
    Signal MEM_USB_RESET_FromUsb    : Std_Logic;

    Signal MEM_D_O_FromFlash         : Std_Logic_Vector(31 Downto 0);
    Signal MEM_D_E_FromFlash         : Std_Logic;
    Signal MEM_A_FromFlash           : Std_Logic_Vector(23 Downto 0);
    Signal MEM_OE_FromFlash          : Std_Logic;
    Signal MEM_W_FromFlash           : Std_Logic;
    Signal MEM_SRAM_E_FromFlash      : Std_Logic;
    Signal MEM_BE_FromFlash          : Std_Logic_Vector(3 Downto 0);
    Signal MEM_SDRAM_CKE_FromFlash   : Std_Logic;
    Signal MEM_SDRAM_RAS_FromFlash   : Std_Logic;
    Signal MEM_SDRAM_CAS_FromFlash   : Std_Logic;
    Signal MEM_SDRAM_E_FromFlash     : Std_Logic;
    Signal MEM_FLASH_E_FromFlash     : Std_Logic;
    Signal MEM_FLASH_RESET_FromFlash : Std_Logic;
    Signal MEM_USB_E_FromFlash         : Std_Logic;
    Signal MEM_USB_RESET_FromFlash   : Std_Logic;

    Signal Resolved_USB_CYC_I  : Std_Logic;
    Signal Resolved_USB_STB_I  : Std_Logic;
    Signal Resolved_FLASH_CYC_I : Std_Logic;
    Signal Resolved_FLASH_STB_I : Std_Logic;
    Signal UsbAcknowledge  : Std_Logic;
    Signal FlashAcknowledge : Std_Logic;


    Type TArbiterState Is (sAccess_Flash, sAccess_Usb);
    Signal ArbiterState      : TArbiterState;
    Signal NextArbiterState  : TArbiterState;
    Signal PossibleNextState : TArbiterState;

    Signal FlashAccessRequest : Std_Logic;
    Signal UsbAccessRequest  : Std_Logic;


    ----------------------------------------------------------------------------
Begin

    ----------------------------------------------------------------------------
SubPart_Usb  : Configurable_ReplaceWith_Designator_Usb Port Map (
     MEM_D_O         => MEM_D_O_FromUsb,
     MEM_D_I         => MEM_D,
     MEM_D_E         => MEM_D_E_FromUsb,
     MEM_A           => MEM_A_FromUsb,
     MEM_OE          => MEM_OE_FromUsb,
     MEM_W           => MEM_W_FromUsb,
     MEM_SRAM_E      => MEM_SRAM_E_FromUsb,
     MEM_BE          => MEM_BE_FromUsb,
     ---------------------------------------------------------------------------
     MEM_SDRAM_CKE   => MEM_SDRAM_CKE_FromUsb,
     MEM_SDRAM_RAS   => MEM_SDRAM_RAS_FromUsb,
     MEM_SDRAM_CAS   => MEM_SDRAM_CAS_FromUsb,
     MEM_SDRAM_E     => MEM_SDRAM_E_FromUsb,
     MEM_FLASH_E     => MEM_FLASH_E_FromUsb,
     MEM_FLASH_BUSY  => MEM_FLASH_BUSY,
     MEM_FLASH_RESET => MEM_FLASH_RESET_FromUsb,
     MEM_USB_E       => MEM_USB_E_FromUsb,
     MEM_USB_RESET   => MEM_USB_RESET_FromUsb,
     ---------------------------------------------------------------------------
     USB_CLK_I      => USB_CLK_I,
     USB_RST_I      => USB_RST_I,
     USB_CYC_I      => Resolved_USB_CYC_I,
     USB_STB_I      => Resolved_USB_STB_I,
     USB_ACK_O      => UsbAcknowledge,
     USB_ADR_I      => USB_ADR_I,
     USB_DAT_I      => USB_DAT_I,
     USB_DAT_O      => USB_DAT_O,
     USB_WE_I       => USB_WE_I,
     USB_SEL_I      => USB_SEL_I
);
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
SubPart_Flash : Configurable_ReplaceWith_Designator_Flash Port Map (
     MEM_D_O         => MEM_D_O_FromFlash,
     MEM_D_I         => MEM_D,
     MEM_D_E         => MEM_D_E_FromFlash,
     MEM_A           => MEM_A_FromFlash,
     MEM_OE          => MEM_OE_FromFlash,
     MEM_W           => MEM_W_FromFlash,
     MEM_FLASH_E     => MEM_FLASH_E_FromFlash,
     MEM_FLASH_BUSY  => MEM_FLASH_BUSY,
     MEM_FLASH_RESET => MEM_FLASH_RESET_FromFlash,
     ---------------------------------------------------------------------------
     MEM_SDRAM_CKE   => MEM_SDRAM_CKE_FromFlash,
     MEM_SDRAM_RAS   => MEM_SDRAM_RAS_FromFlash,
     MEM_SDRAM_CAS   => MEM_SDRAM_CAS_FromFlash,
     MEM_SDRAM_E     => MEM_SDRAM_E_FromFlash,
     MEM_SRAM_E      => MEM_SRAM_E_FromFlash,
     MEM_BE          => MEM_BE_FromFlash,
     MEM_USB_E       => MEM_USB_E_FromFlash,
     MEM_USB_RESET   => MEM_USB_RESET_FromFlash,
     ---------------------------------------------------------------------------
     FLASH_CLK_I     => FLASH_CLK_I,
     FLASH_RST_I     => FLASH_RST_I,
     FLASH_CYC_I     => Resolved_FLASH_CYC_I,
     FLASH_STB_I     => Resolved_FLASH_STB_I,
     FLASH_ACK_O     => FlashAcknowledge,
     FLASH_ADR_I     => FLASH_ADR_I,
     FLASH_DAT_I     => FLASH_DAT_I,
     FLASH_DAT_O     => FLASH_DAT_O,
     FLASH_WE_I      => FLASH_WE_I,
     FLASH_SEL_I     => FLASH_SEL_I
);
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    MEM_A              <= MEM_A_FromUsb           When MEM_USB_E_FromUsb='0' Else MEM_A_FromFlash;
    MEM_OE             <= MEM_OE_FromUsb          When MEM_USB_E_FromUsb='0' Else MEM_OE_FromFlash;
    MEM_W              <= MEM_W_FromUsb           When MEM_USB_E_FromUsb='0' Else MEM_W_FromFlash;
    MEM_SDRAM_CKE      <= MEM_SDRAM_CKE_FromUsb   When MEM_USB_E_FromUsb='0' Else MEM_SDRAM_CKE_FromFlash;
    MEM_SDRAM_RAS      <= MEM_SDRAM_RAS_FromUsb   When MEM_USB_E_FromUsb='0' Else MEM_SDRAM_RAS_FromFlash;
    MEM_SDRAM_CAS      <= MEM_SDRAM_CAS_FromUsb   When MEM_USB_E_FromUsb='0' Else MEM_SDRAM_CAS_FromFlash;
    MEM_SDRAM_E        <= MEM_SDRAM_E_FromUsb     When MEM_USB_E_FromUsb='0' Else MEM_SDRAM_E_FromFlash;
    MEM_FLASH_E        <= MEM_FLASH_E_FromUsb     When MEM_USB_E_FromUsb='0' Else MEM_FLASH_E_FromFlash;
    MEM_FLASH_RESET    <= MEM_FLASH_RESET_FromUsb When MEM_USB_E_FromUsb='0' Else MEM_FLASH_RESET_FromFlash;
    MEM_SRAM_E         <= MEM_SRAM_E_FromUsb      When MEM_USB_E_FromUsb='0' Else MEM_SRAM_E_FromFlash;
    MEM_BE             <= MEM_BE_FromUsb          When MEM_USB_E_FromUsb='0' Else MEM_BE_FromFlash;
    MEM_USB_E          <= MEM_USB_E_FromUsb       When MEM_USB_E_FromUsb='0' Else MEM_USB_E_FromFlash;
    MEM_USB_RESET      <= MEM_USB_RESET_FromUsb   When MEM_USB_E_FromUsb='0' Else MEM_USB_RESET_FromFlash;

    MEM_D              <= MEM_D_O_FromUsb           When MEM_D_E_FromUsb  ='1' Else
                          MEM_D_O_FromFlash         When MEM_D_E_FromFlash='1' Else
                          (Others=>'Z');


    FlashAccessRequest <= FLASH_CYC_I And FLASH_STB_I;
    UsbAccessRequest  <= USB_CYC_I  And USB_STB_I;
    ----------------------------------------------------------------------------
    Process(FlashAccessRequest)
    Begin
         If FlashAccessRequest='1' Then PossibleNextState <= sAccess_Flash;
                                   Else PossibleNextState <= sAccess_Usb;
         End If;
    End Process;
    ----------------------------------------------------------------------------

    ----------------------------------------------------------------------------
    Process(ArbiterState, FLASH_STB_I, FlashAcknowledge, FLASH_CYC_I, USB_STB_I, USB_CYC_I, UsbAcknowledge, FlashAccessRequest, PossibleNextState)
    Begin
         NextArbiterState <= ArbiterState;
         Resolved_FLASH_STB_I <= '0';
         Resolved_FLASH_CYC_I <= '0';
         Resolved_USB_STB_I  <= '0';
         Resolved_USB_CYC_I  <= '0';
         FLASH_ACK_O          <= '0';
         USB_ACK_O           <= '0';

         Case ArbiterState Is
              When sAccess_Flash =>
                   If FLASH_STB_I='0' Then
                      NextArbiterState <= PossibleNextState;
                   ElsIf FlashAcknowledge='1' Then
                   End If;
                   Resolved_FLASH_STB_I <= FLASH_STB_I;
                   Resolved_FLASH_CYC_I <= FLASH_CYC_I;
                   FLASH_ACK_O          <= FlashAcknowledge;

              When sAccess_Usb  =>
                   If USB_STB_I='0' Then
                      NextArbiterState <= PossibleNextState;
                   ElsIf UsbAcknowledge='1' Then
                         If FlashAccessRequest='1' Then
                            NextArbiterState <= PossibleNextState;
                         End If;
                   End If;
                   Resolved_USB_STB_I <= USB_STB_I;
                   Resolved_USB_CYC_I <= USB_CYC_I;
                   USB_ACK_O          <= UsbAcknowledge;

              When Others       =>
                   NextArbiterState <= sAccess_Flash;
         End Case;
    End Process;
    ----------------------------------------------------------------------------


    ----------------------------------------------------------------------------
    Process(FLASH_CLK_I)
    Begin
         If Rising_Edge(FLASH_CLK_I) Then
            If FLASH_RST_I='1' Then
                ArbiterState <= sAccess_Flash;
            Else
                ArbiterState <= NextArbiterState;
            End If;
         End If;
    End Process;
    ----------------------------------------------------------------------------





    ----------------------------------------------------------------------------
End RTL;
